#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Contains constants which are used across SDK modules."""
from __future__ import annotations

from enum import Enum, auto
from logging import Formatter
from typing import Any, List, Literal, TypedDict

MB = 2**20
MAX_ARTIFACT_SIZE_MB: int = 10
DOWNLOAD_CHUNK_SIZE = 128

LOG_FORMAT = Formatter("%(asctime)s %(name)s %(levelname)s: %(message)s")
LOG_LEVELS = Literal["CRITICAL", "FATAL", "ERROR", "WARNING", "WARN", "INFO", "DEBUG"]


class _ArtifactGlob(TypedDict, total=False):
    ignore_files: list[str]


class ArtifactInfo(_ArtifactGlob):
    """TypedDict with all required fields for artifact creation.

    path: Path to the file or folder that will be compressed and used as artifact.
    ignore_files: Ignore files to be used when creating artifact.
    """

    path: str


class AutoName(Enum):
    """Enum with automatic name() values."""

    @staticmethod
    def _generate_next_value_(name: str, start: int, count: int, last_values: List[Any]) -> str:  # noqa: ARG004
        """Automatically generate enum values from names."""
        return str(name).lower()


class Stage(AutoName):
    """Enum of all supported platform stages."""

    DEV = auto()
    LATEST = auto()
    TEST = auto()
    BETA = auto()
    PROD = auto()
    PARVATI = auto()


class ContentType(Enum):
    """Enum of supported content type for http request to API."""

    APPLICATION_JSON = "application/json"
    MULTIPART_FORM_DATA = "multipart/form-data"


class HttpMethods(AutoName):
    """Enum of supported HTTP methods."""

    GET = auto()
    POST = auto()
    PUT = auto()
    PATCH = auto()
    DELETE = auto()


class Sources(AutoName):
    """Enum of the sources for telemetry call."""

    SDK = auto()
    CLI = auto()


class OutputTypes(str, Enum):
    """Enum of the Output Types allowed in CLI commands."""

    json = "json"
    yaml = "yaml"
    table = "table"


class OutputTypesNoTable(str, Enum):
    """Enum of the Output Types allowed in CLI commands without the table."""

    json = "json"
    yaml = "yaml"


class OutputTypesOnlyJson(str, Enum):
    """Enum of the Output Types allowed in CLI commands without the table."""

    json = "json"


__all__: List[str] = [
    "MB",
    "MAX_ARTIFACT_SIZE_MB",
    "DOWNLOAD_CHUNK_SIZE",
    "LOG_FORMAT",
    "ArtifactInfo",
    "Stage",
    "ContentType",
    "HttpMethods",
    "LOG_LEVELS",
    "Sources",
    "OutputTypes",
    "OutputTypesNoTable",
    "OutputTypesOnlyJson",
]



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Peak blocks specs commands."""

from typing import Dict, List, Optional

import typer
from peak.cli import args, helpers
from peak.cli.args import DRY_RUN, GENERATE_YAML, OUTPUT_TYPES, PAGING
from peak.constants import OutputTypes, OutputTypesNoTable
from peak.output import Writer
from peak.press.blocks import Block
from rich.console import Console

app = typer.Typer()
console = Console()

_SPEC_ID = typer.Argument(..., help="ID of the Block spec to be used in this operation.")


@app.command("list", short_help="List Block specs.")
def list_block_specs(
    ctx: typer.Context,
    page_size: Optional[int] = args.PAGE_SIZE,
    page_number: Optional[int] = args.PAGE_NUMBER,
    status: Optional[List[str]] = args.STATUS_FILTER_SPECS,
    name: Optional[str] = args.NAME_FILTER,
    title: Optional[str] = args.TITLE_FILTER,
    sort: Optional[List[str]] = args.SORT_KEYS,
    scope: Optional[List[str]] = args.SCOPES,
    featured: Optional[bool] = args.FEATURED,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** all Block specs that exists in the tenant along with the public-scoped ones.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak blocks specs list --featured --scope shared,private --page-size 10 --page-number 1
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "specsCount": 1,
        "specs": [...],
        "pageCount": 1,
        "pageNumber": 1,
        "pageSize": 10
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/get_v1_blocks_specs)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = block_client.list_specs(
            status=status,
            featured=featured,
            name=name,
            title=title,
            sort=sort,
            scope=scope,
            page_size=page_size,
            page_number=page_number,
            return_iterator=False,
        )
        writer.write(response)


@app.command(short_help="Create a Block spec.")
def create(
    ctx: typer.Context,
    file: str = args.TEMPLATE_PATH,
    description_file: Optional[str] = args.TEMPLATE_DESCRIPTION_FILE,
    release_notes_file: Optional[str] = args.RELEASE_NOTES_FILE,
    params_file: Optional[str] = args.TEMPLATE_PARAMS_FILE,
    params: Optional[List[str]] = args.TEMPLATE_PARAMS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Create*** a Block spec. A Block spec is just a blueprint for a specific resource. This creates a version of the spec also know as a ***Release***.

    \b
    🧩 ***Input file schema (yaml):***<br/>
    ```yaml
    featured (bool | required: false): Boolean specifying whether to feature this spec.
    scope (str | required: false): Specify what tenants can discover and deploy this spec.
    tenants (list(str) | required: false): Given a shared scope, specify what other tenants can discover and deploy this spec.
    autoRunOnDeploy (bool | required: false): Whether to execute the resource after the block is deployed. By default it is False.
    body (map):
        version (int): Version of the spec.
        kind (string): Specifies the type of spec.
        metadata (map):
            name (string): Name of the spec. Must be unique within the tenant.
            title (string | required: false): Title of the spec.
            summary (string): Summary of the spec.
            description (string | required: false): Description of the spec.
            descriptionContentType (string | required: false): Content type of the description. Should be one of "text/plain" or "text/markdown".
            imageUrl (string | required: false): URL of the image to be associated with the app spec.
            tags (list(map) | required: false):
                - name (string): Name of the tag.
        release (map):
            version (string): A valid semantic release version of the spec.
            notes (string | required: false): Notes for the release version.
        # Workflow Block
        config (map):
            images(map | required: false):
                image-name (map): Dictionary containing the image configuration. Here the key is name of the image.
                    version (string | required: false): A valid semantic image version.
                    dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                    context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                    useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                    buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                    secrets (list(str)) | required: false): List of secret names to be passed.
            steps (map | required: false):
                <stepName> (map): Dictionary containing the step configuration. Here the key is name of the step.
                    # Standard Step
                    image: (map | required: false):
                        version (string | required: false): A valid semantic image version.
                        dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                        context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                        useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                        buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                        secrets (list(str)) | required: false): List of secret names to be passed.
                    imageDetails (map | required: false):
                        id (int): ID of the existing image.
                        versionId: (int): ID of the existing image version.
                    imageRef (string | required: false): Name of the image defined above.
                    type (string | required: false): Type of workflow step. This should be 'standard' for a Standard Step which is the default value.
                    command (string): Command to run when step is executed.
                    clearImageCache (boolean | required: false): Whether to clear image cache on workflow execution.
                    stepTimeout (int | required: false): Time after which the step timeouts.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        secrets (list(str) | required: false): List of secret names to be passed.
                    parents (list(str) | required: false): List containing names of steps on which this step is dependent on.
                    repository (map | required: false):
                        branch (string): Branch of the repository containing the required files.
                        token (string | required: false): The token to be used to clone the repository.
                        url (string): URL of the repository.
                    resources (map | required: false):
                        instanceTypeId (int): ID of the instance type to be used in the step.
                        storage (string): Storage in GB. For example, "10GB".
                    outputParameters (map | required: false):
                        <keyName> (string | required: false): Represents a key-value pair where the key is the parameter name, and the value is the file name where the output is stored.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # HTTP Step
                    type (string | required: false): Type of workflow step. This should be 'http' for a HTTP Step.
                    method (string | required: true): The HTTP method to be used for the API call. Should be one of - get, post, put, patch, and delete
                    url (string | required: true): The URL to make the HTTP API call to.
                    payload (string | required: false): Stringified JSON payload to be sent in case of non-get requests.
                    headers (map | required: false):
                        absolute (map | required: false): Key-Value pair where key is the name of the header and value is the value of the header.
                        secrets (map | required: false): Key-Value pair where key is the name of the header and value is the name of the secret to get the value from.
                    auth (map | required: false):
                        type (string | required: true): The type of authentication to use. Should be one of - no-auth, oauth, basic, api-key, and bearer-token.
                        clientId (string | required: false): The Client ID used to get the access token in case of OAuth. Value should be the name of a secret. Required when type is oauth.
                        clientSecret (string | required: false): The Client Secret used to get the access token in case of OAuth. Value should be the name of a secret. Required when type is oauth.
                        authUrl (string | required: false): The URL that is hit to get the access token in case of OAuth. Required when type is oauth.
                        username (string | required: false): The username used for basic authentication. Required when type is basic.
                        password (string | required: false): The password used for basic authentication. Value should be the name of a secret. Required when type is basic.
                        apiKey (string | required: false): The API Key used for authentication. Value should be the name of a secret. Required when type is api-key.
                        bearerToken (string | required: false): The bearer token used for authentication. Value should be the name of a secret. Required when type is bearer-token.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        inherit (map | required: false): Key-Value pair where key is the name of the env and value is the name of the output parameter.
                        secrets (list(str) | required: false): List of secret names to be passed.
                    outputParameters (map | required: false):
                        <keyName> (string | required: false): Represents a key-value pair where the key is the parameter name. Note that in case of HTTP step, value should always be "response.txt", otherwise this parameter would be rejected.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # Export Step
                    type (string | required: true): Type of workflow step. This should be 'export' for a export Step.
                    schema (string | required: true): Schema from which the data is to be exported.
                    table (string | required: true): Table from which the data is to be exported.
                    sortBy (string | required: true): Column name by which the data is to be sorted.
                    sortOrder (string | required: false): Order in which the data is to be sorted. Should be one of - asc, desc. Default is asc.
                    compression (boolean | required: false): Whether to compress the files to '.gz' format while exporting. Default is false.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # SQL Step
                    type (string | required: true): Type of workflow step. This should be 'sql' for a sql Step.
                    sqlQueryPath (string | required: false): Path to the SQL query file. One of the 'sqlQueryPath' or 'repository' is required.
                    repository (map | required: false):
                        branch (string | required: true): Branch of the repository containing the required files.
                        token (string | required: false): The token to be used to clone the repository.
                        url (string | required: true): URL of the repository.
                        filePath (string | required: true): Path to the file containing the SQL query.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        inherit (map | required: false): Key-Value pair where key is the name of the env and value is the name of the output parameter.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
            triggers (list(map)):
                - cron (string | required: false): A valid cron expression.
                  webhook (boolean | required: false): Should be true if webhook type trigger is to be used.
                  webhookId (string | required: false): ID of the webhook.
            watchers (list(map) | required: false):
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  user (string): User to be notified.
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  webhook (map):
                    name (string): Name of the webhook.
                    url (string): URL of the webhook.
                    payload (string): Webhook payload.
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  email (map):
                    name (string): Name of the email watcher.
                    recipients (map):
                        to (list(str)): List of email addresses to send the email to. Email can be sent only to the users who are added in the tenant.
        # Service Block
        config (map): # One of "image" or "imageDetails" is required.
            serviceType (string): Type of the service. Should be one of "web-app" or "api". It is "web-app" by default.
            image (map | required: false):
                version (string | required: false): A valid semantic image version.
                dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                secrets (list(str)) | required: false): List of secret names to be passed.
            imageDetails (map | required: false):
                id (int): ID of the existing image.
                versionId: (int | required: false): ID of the existing image version. If not provided, the latest version will be used.
            resources (map | required: false): To be used only in case of service block.
                instanceTypeId (int): ID of the instance type to be used in the service block.
            parameters (map | required: false):
                env (map | required: false): Key-Value pair where key is the name of the env.
                secrets (list(str) | required: false): List of secret names to be passed.
            sessionStickiness (boolean | required: false): Whether to enable session stickiness for the service. It is false by default. To be used only in case of service block of type web-app.
            entrypoint (string | required: false): Entry point for the service.
            healthCheckURL (string | required: false): URL to check the health of the service.
            minInstances (number | required: false): Minimum number of instances that would run for a service. Default value is 1 and maximum value allowed is 2.
    parameters (map | required: false):
        <parameterType> (list(map)): List containing the parameter objects. Here the key is the type of the parameter. Accepted values are "build" or "run".
            name (string): Name of the parameter.
            type (string): Type of the parameter. Should be one of "boolean", "string", "string_array", "number", "number_array", "object" and "object_array".
            required (boolean): Whether the parameter is required.
            description (string | required: false): Description of the parameter.
            defaultValue (string | required: false): Default value of the parameter.
            title (string | required: false): Title of the parameter.
            options (list(str) | required: false): List of options for the parameter. If provided, it must have at least one object with "title" and "value" field.
            hideValue(boolean | required: false): Can be optionally provided to parameters of type "string", to mask the parameter's value when it has been set at deployment time.
    artifact (map | required: false):
      path (str): Path to the artifact.
      ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak blocks specs create /path/to/body.yaml -v /path/to/params.yaml
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "id": "632a4e7c-ab86-4ecb-8f34-99b5da531ceb"
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/post_v1_blocks_specs)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    markdown_files = {}
    if description_file:
        markdown_files["body:metadata:description"] = description_file
    if release_notes_file:
        markdown_files["body:release:notes"] = release_notes_file

    body = helpers.template_handler(file, params_file, params, markdown_files)
    body = helpers.remove_unknown_args(body, block_client.create_spec)

    with writer.pager():
        response: Dict[str, str] = block_client.create_spec(**body)
        writer.write(response)


@app.command(short_help="Describe a Block spec.")
def describe(
    ctx: typer.Context,
    spec_id: str = _SPEC_ID,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Describe*** a Block spec with details of its latest release.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak blocks specs describe "632a4e7c-ab86-4ecb-8f34-99b5da531ceb"
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "featured": true,
        "id": "632a4e7c-ab86-4ecb-8f34-99b5da531ceb"
        "kind": "app",
        "latestRelease": {...},
        "metadata": {...},
        "parameters": {...},
        "scope": "private"
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/get_v1_blocks_specs__specId_)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = block_client.describe_spec(spec_id)
        writer.write(response)


@app.command(short_help="Update the Block spec metadata.")
def update_metadata(
    ctx: typer.Context,
    spec_id: str = _SPEC_ID,
    file: str = args.TEMPLATE_PATH,
    description_file: Optional[str] = args.TEMPLATE_DESCRIPTION_FILE,
    params_file: Optional[str] = args.TEMPLATE_PARAMS_FILE,
    params: Optional[List[str]] = args.TEMPLATE_PARAMS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Update*** the Block spec metadata.

    \b
    🧩 ***Input file schema (yaml):***<br/>
    ```yaml
      body (map):
        featured (bool | required: false): Boolean specifying whether to feature this spec.
        scope (str | required: false): Specify what tenants can discover and deploy this spec.
        tenants (list(str) | required: false): Given a shared scope, specify what other tenants can discover and deploy this spec.
        metadata (map):
            name (string | required: false): Name of the spec. Must be unique within the tenant.
            title (string | required: false): Title of the spec.
            summary (string | required: false): Summary of the spec.
            description (string | required: false): Description of the spec.
            descriptionContentType (string | required: false): Content type of the description. Should be one of "text/plain" or "text/markdown".
            imageUrl (string | required: false): URL of the image to be associated with the block spec.
            status (string | required: false): Status of the block spec.
            tags (list(map) | required: false):
                - name (string): Name of the tag.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak blocks specs update-metadata "632a4e7c-ab86-4ecb-8f34-99b5da531ceb" /path/to/body.yaml -v /path/to/params.yaml
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "featured": false,
        "id": "632a4e7c-ab86-4ecb-8f34-99b5da531ceb"
        "kind": "workflow",
        "latestRelease": {...},
        "metadata": {...},
        "scope": "private"
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/patch_v1_blocks_specs__specId_)
    """
    markdown_files = {}
    if description_file:
        markdown_files["body:metadata:description"] = description_file

    body = helpers.template_handler(file, params_file, params, markdown_files)
    writer: Writer = ctx.obj["writer"]

    block_client: Block = ctx.obj["client"]
    body = helpers.remove_unknown_args(body, block_client.update_spec_metadata)

    with writer.pager():
        response: Dict[None, None] = block_client.update_spec_metadata(spec_id, **body)
        writer.write(response)


@app.command(short_help="Delete a Block spec.")
def delete(
    ctx: typer.Context,
    spec_id: str = _SPEC_ID,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Delete*** a Block spec.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak blocks specs delete "632a4e7c-ab86-4ecb-8f34-99b5da531ceb"
    ```

    \b
    🆗 ***Response:***
    ```json
    {}
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/delete_v1_blocks_specs__specId_)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = block_client.delete_spec(spec_id)
        writer.write(response)


@app.command(short_help="Create a new release for a Block spec.")
def create_release(
    ctx: typer.Context,
    spec_id: str = _SPEC_ID,
    file: str = args.TEMPLATE_PATH,
    release_notes_file: Optional[str] = args.RELEASE_NOTES_FILE,
    params_file: Optional[str] = args.TEMPLATE_PARAMS_FILE,
    params: Optional[List[str]] = args.TEMPLATE_PARAMS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Create*** a new release for a Block spec.

    \b
    🧩 ***Input file schema (yaml):***<br/>
    ```yaml
    body (map):
        release (map):
            version (string): A valid semantic release version of the spec. Must be greater than previous release version.
            notes (string | required: false): Notes for the release version.
        # Workflow Block
        config (map):
            images(map | required: false):
                image-name(map): Dictionary containing the image configuration. Here the key is name of the image.
                    version (string | required: false): A valid semantic image version.
                    dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                    context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                    useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                    buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                    secrets (list(str)) | required: false): List of secret names to be passed.
            steps(map | required: false):
                <stepName> (map): # Dictionary containing the step configuration. Here the key is name of the step.
                    # Standard Step
                    image: (map | required: false):
                        version (string | required: false): A valid semantic image version.
                        dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                        context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                        useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                        buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                        secrets (list(str)) | required: false): List of secret names to be passed.
                    imageDetails (map | required: false):
                        id (int): ID of the existing image.
                        versionId: (int): ID of the existing image version.
                    imageRef (string | required: false): Name of the image defined above.
                    type (string | required: false): Type of workflow step. This should be 'standard' for a Standard Step which is the default value.
                    command (string): Command to run when step is executed.
                    clearImageCache (boolean | required: false): Whether to clear image cache on workflow execution.
                    stepTimeout (int | required: false): Time after which the step timeouts.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        inherit (map | required: false): Key-Value pair where key is the name of the env and value is the name of the output parameter.
                        secrets (list(str) | required: false): List of secret names to be passed.
                    outputParameters (map | required: false):
                        <keyName> (string | required: false): Represents a key-value pair where the key is the parameter name, and the value is the file name where the output is stored.
                    parents (list(str) | required: false): List containing names of steps on which this step is dependent on.
                    repository (map | required: false):
                        branch (string): Branch of the repository containing the required files.
                        token (string | required: false): The token to be used to clone the repository.
                        url (string): URL of the repository.
                    resources (map | required: false):
                        instanceTypeId (int): ID of the instance type to be used in the step.
                        storage (string): Storage in GB. For example, "10GB".
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # HTTP Step
                    type (string | required: false): Type of workflow step. This should be 'http' for a HTTP Step.
                    method (string | required: true): The HTTP method to be used for the API call. Should be one of - get, post, put, patch, and delete
                    url (string | required: true): The URL to make the HTTP API call to.
                    payload (string | required: false): Stringified JSON payload to be sent in case of non-get requests.
                    headers (map | required: false):
                        absolute (map | required: false): Key-Value pair where key is the name of the header and value is the value of the header.
                        secrets (map | required: false): Key-Value pair where key is the name of the header and value is the name of the secret to get the value from.
                    auth (map | required: false):
                        type (string | required: true): The type of authentication to use. Should be one of - no-auth, oauth, basic, api-key, and bearer-token.
                        clientId (string | required: false): The Client ID used to get the access token in case of OAuth. Value should be the name of a secret. Required when type is oauth.
                        clientSecret (string | required: false): The Client Secret used to get the access token in case of OAuth. Value should be the name of a secret. Required when type is oauth.
                        authUrl (string | required: false): The URL that is hit to get the access token in case of OAuth. Required when type is oauth.
                        username (string | required: false): The username used for basic authentication. Required when type is basic.
                        password (string | required: false): The password used for basic authentication. Value should be the name of a secret. Required when type is basic.
                        apiKey (string | required: false): The API Key used for authentication. Value should be the name of a secret. Required when type is api-key.
                        bearerToken (string | required: false): The bearer token used for authentication. Value should be the name of a secret. Required when type is bearer-token.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        inherit (map | required: false): Key-Value pair where key is the name of the env and value is the name of the output parameter.
                        secrets (list(str) | required: false): List of secret names to be passed.
                    outputParameters (map | required: false):
                        <keyName> (string | required: false): Represents a key-value pair where the key is the parameter name. Note that in case of HTTP step, value should always be "response.txt", otherwise this parameter would be rejected.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # Export Step
                    type (string | required: true): Type of workflow step. This should be 'export' for a export Step.
                    schema (string | required: true): Schema from which the data is to be exported.
                    table (string | required: true): Table from which the data is to be exported.
                    sortBy (string | required: true): Column name by which the data is to be sorted.
                    sortOrder (string | required: false): Order in which the data is to be sorted. Should be one of - asc, desc. Default is asc.
                    compression (boolean | required: false): Whether to compress the files to '.gz' format while exporting. Default is false.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
                    # SQL Step
                    type (string | required: true): Type of workflow step. This should be 'sql' for a sql Step.
                    sqlQueryPath (string | required: false): Path to the SQL query file. One of the 'sqlQueryPath' or 'repository' is required.
                    repository (map | required: false):
                        branch (string | required: true): Branch of the repository containing the required files.
                        token (string | required: false): The token to be used to clone the repository.
                        url (string | required: true): URL of the repository.
                        filePath (string | required: true): Path to the file containing the SQL query.
                    parameters (map | required: false):
                        env (map | required: false): Key-Value pair where key is the name of the env.
                        inherit (map | required: false): Key-Value pair where key is the name of the env and value is the name of the output parameter.
                    executionParameters (map | required: false):
                        conditional (list(map) | required: false): # List of conditions to be evaluated based on the output params of the parent steps.
                            - condition (string): The evaluation condition, either 'equals' or 'not-equals'.
                            paramName (string): Name of the output parameter to be evaluated.
                            stepName (string): Name of the step containing the output parameter.
                            value (string): Value to be compared with the output parameter.
                        parentStatus (list(map) | required: false): # List of conditions to be evaluated based on the status of the parent steps.
                            - condition (string): The evaluation condition, either 'all-of' or 'one-of'.
                            parents (list(str)): List of parent step names.
                            status (list(str)): List of parent step statuses, with valid values being 'success' and 'failure'.
                    runConfiguration (map | required: false):
                        retryOptions (map | required: false): # Step level retry options which will override the workflow level retry options.
                            duration (int | required: false): Duration in seconds after which the step is retried if it fails. Default is 5 seconds and maximum is 120 seconds.
                            exitCodes (list(int) | required: false): List of exit codes for which the step is retried. If not provided, the step is retried for all exit codes.
                            exponentialBackoff (boolean | required: false): Whether to use exponential backoff for retrying the step. If provided as true, the factor used will be 2.
                            numberOfRetries (int | required: false): Number of times the step should be retried. Default is 3 and maximum is 5.
                        skipConfiguration (map | required: false):
                            skip (boolean | required: false): Indicates whether to skip the step. Default is false.
                            skipDAG (boolean | required: false): Indicates whether to skip dependent steps, including the current step. Default is false.
            triggers (list(map)):
                cron (string | required: false): A valid cron expression.
                webhook (boolean | required: false): Should be true if webhook type trigger is to be used.
                webhookId (string | required: false): ID of the webhook.
            watchers (list(map) | required: false):
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  user (string): User to be notified.
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  webhook (map):
                    name (string): Name of the webhook.
                    url (string): URL of the webhook.
                    payload (string): Webhook payload.
                - events (map):
                    success (boolean | required: false): Whether to call event on success.
                    fail (boolean | required: false): Whether to call event on failure.
                    runtimeExceeded (int | required: false): The runtime in minutes after which event is called.
                  email (map):
                    name (string): Name of the email watcher.
                    recipients (map):
                        to (list(str)): List of email addresses to send the email to. Email can be sent only to the users who are added in the tenant.
        # Service Block
        config (map): # One of "image" or "imageDetails" is required.
            serviceType (string): Type of the service. Should be one of "web-app" or "api". It is "web-app" by default.
            image (map | required: false):
                version (string | required: false): A valid semantic image version.
                dockerfile (string | required: false): Path to the Dockerfile inside artifact.
                context (string | required: false): The path within the artifact where the code to be executed by the Dockerfile is located.
                useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
                buildArguments (map | required: false): Dictionary containing build args. Here the key is the name of the arg and value is the value of the arg.
                secrets (list(str)) | required: false): List of secret names to be passed.
            imageDetails (map | required: false):
                id (int): ID of the existing image.
                versionId: (int | required: false): ID of the existing image version. If not provided, the latest version will be used.
            resources (map | required: false):
                instanceTypeId (int): ID of the instance type to be used in the service block.
            parameters (map | required: false):
                env (map | required: false): Key-Value pair where key is the name of the env.
                secrets (list(str) | required: false): List of secret names to be passed.
            sessionStickiness (boolean | required: false): Whether to enable session stickiness for the service. It is false by default. To be used only in case of service block of type web-app.
            entrypoint (string | required: false): Entry point for the service.
            healthCheckURL (string | required: false): URL to check the health of the service.
            minInstances (number | required: false): Minimum number of instances that would run for a service. Default value is 1 and maximum value allowed is 2.
    parameters (map | required: false):
        <parameterType> (list(map)): List containing the parameter objects. Here the key is the type of the parameter. Accepted values are "build" or "run".
            name (string): Name of the parameter.
            type (string): Type of the parameter. Should be one of "boolean", "string", "string_array", "number", "number_array", "object" and "object_array".
            required (boolean): Whether the parameter is required.
            description (string | required: false): Description of the parameter.
            defaultValue (string | required: false): Default value of the parameter.
            title (string | required: false): Title of the parameter.
            options (list(str) | required: false): List of options for the parameter. If provided, it must have at least one object with "title" and "value" field.
    artifact (map | required: false):
        path (str): Path to the artifact.
        ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    autoRunOnDeploy (bool | required: false): Whether to execute the resource after the block is deployed. By default it is False.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak blocks specs create-release "632a4e7c-ab86-4ecb-8f34-99b5da531ceb" '/path/to/body.yaml' -v '/path/to/params.yaml'
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "id": "632a4e7c-ab86-4ecb-8f34-99b5da531ceb",
        "release": {
            "version": "2.0.0"
        }
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/post_v1_blocks_specs__specId__releases)
    """
    markdown_files = {}
    if release_notes_file:
        markdown_files["body:release:notes"] = release_notes_file

    body = helpers.template_handler(file, params_file, params, markdown_files)
    writer: Writer = ctx.obj["writer"]

    block_client: Block = ctx.obj["client"]
    body = helpers.remove_unknown_args(body, block_client.create_spec_release)

    with writer.pager():
        response: Dict[str, str] = block_client.create_spec_release(spec_id, **body)
        writer.write(response)


@app.command(short_help="Describe a Block spec release.")
def describe_release(
    ctx: typer.Context,
    spec_id: str = args.SPEC_ID,
    version: str = args.RELEASE_VERSION,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Describe*** a Block spec release.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak blocks specs describe-release --spec-id "632a4e7c-ab86-4ecb-8f34-99b5da531ceb" --version 1.0.0
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "artifact": {
            "id": "721d738a-29f3-43b2-af52-c9055abe60b6",
            "version": 1
        },
        "config": {
            "images": {...},
            "steps": {...},
            "triggers": [
                {...}
            ],
            "watchers": [
                {...}
            ]
        },
        "createdAt": "2020-01-01T18:00:00.000Z",
        "createdBy": "someoneh@peak.ai",
        "id": "df113d64-ff44-4aa0-9278-edb03dae7a3f",
        "notes": "This is the original release",
        "autoRunOnDeploy": false,
        "parameters": {...},
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/get_v1_blocks_specs__specId__releases__release_)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = block_client.describe_spec_release(spec_id, version)
        writer.write(response)


@app.command(short_help="List releases of a Block spec.")
def list_releases(
    ctx: typer.Context,
    spec_id: str = args.SPEC_ID,
    sort: Optional[List[str]] = args.SORT_KEYS,
    page_size: Optional[int] = args.PAGE_SIZE,
    page_number: Optional[int] = args.PAGE_NUMBER,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** all releases for a given Block spec.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak blocks specs list-releases --spec-id "632a4e7c-ab86-4ecb-8f34-99b5da531ceb" --sort createdBy:asc,createdAt --page-size 10 --page-number 1
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "pageCount": 1,
        "pageNumber": 1,
        "pageSize": 10,
        "releaseCount": 1,
        "releases": [...]
    }
    ```

    🔗 [**API Documentation**](https://press.peak.ai/api-docs/index.htm#/Block%20Specs/get_v1_blocks_specs__specId__releases)
    """
    block_client: Block = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = block_client.list_spec_releases(
            spec_id,
            sort=sort,
            page_size=page_size,
            page_number=page_number,
            return_iterator=False,
        )
        writer.write(response)



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Tenants client module."""
from __future__ import annotations

from typing import Any, Dict, List, Optional

from peak.base_client import BaseClient
from peak.constants import ContentType, HttpMethods
from peak.session import Session


class Tenant(BaseClient):
    """Tenant client class."""

    QUOTA_BASE_ENDPOINT = "quota/api/v1"
    CONNECTIONS_BASE_ENDPOINT = "connections/api/v1"

    def list_instance_options(
        self,
        entity_type: str,
    ) -> Dict[str, Any]:
        """Retrieve details of allowed instance options for a tenant.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/quota/api-docs/index.htm#/settings/get_api_v1_settings_tenant_instance_options>`__

        Args:
            entity_type (str): The type of the entity.
                Allowed values are - api-deployment, data-bridge, webapp, workflow, workspace.

        Returns:
            Dict[str, Any]: a dictionary containing the details of the allowed instance options of a tenant.

        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            NotFoundException: The given image does not exist.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.GET, f"{self.QUOTA_BASE_ENDPOINT}/settings/tenant-instance-options"
        params = {"entityType": entity_type}

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
            params=params,
        )

    def get_credentials(
        self,
        data_store_type: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Retrieve credentials for a given data store type.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/connections/api-docs/index.htm#/connections/get_api_v1_connections_credentials>`__

        Args:
            data_store_type (str): The type of the data store.
                Allowed values are - data-warehouse.
                Default - data-warehouse

        Returns:
            Dict[str, Any]: a dictionary containing the credentials for the data store.

        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            NotFoundException: The given image does not exist.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.GET, f"{self.CONNECTIONS_BASE_ENDPOINT}/connections/credentials"
        params = {"type": data_store_type}

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
            params=params,
        )


def get_client(session: Optional[Session] = None) -> Tenant:
    """Returns a Tenant client, If no session is provided, a default session is used.

    Args:
        session (Optional[Session]): A Session Object. Default is None.

    Returns:
        Tenant: the tenant client object
    """
    return Tenant(session)


__all__: List[str] = ["get_client"]



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Blocks Module."""
import typer
from peak.cli.press.blocks import deployments, specs

app = typer.Typer(
    help="Create and manage Block specs and deployments.",
    short_help="Create and manage Block specs and deployments.",
)
app.add_typer(
    specs.app,
    name="specs",
    help="Create and manage Block specs which are blueprint for Peak resources.",
    short_help="Create and manage Block Specs.",
)
app.add_typer(
    deployments.app,
    name="deployments",
    help="Create and manage Block deployments which deploys the Peak resources based on the created specs.",
    short_help="Create and manage Block Deployments.",
)



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Peak Images service commands."""
from typing import Any, Dict, List, Optional

import typer
from peak.cli import args, helpers
from peak.cli.args import DRY_RUN, GENERATE_YAML, OUTPUT_TYPES, PAGING
from peak.constants import OutputTypes, OutputTypesNoTable, OutputTypesOnlyJson
from peak.helpers import combine_dictionaries, map_user_options, parse_list_of_strings, variables_to_dict
from peak.output import Writer
from peak.resources.images import Image
from typing_extensions import Annotated

app = typer.Typer(
    help="Create Docker images that Workflows, Workspaces, and other services use.",
    short_help="Create and manage Images.",
)

_IMAGE_ID = typer.Argument(..., help="ID of the image to be used in this operation.")

_BUILD_ID = typer.Option(..., help="ID of the image build to be used in this operation.")

_COUNT = typer.Option(None, help="Number of builds required.")

_BUILD_STATUS = typer.Option(
    None,
    help="List of build status to filter image builds. Valid values are `building`, `failed`, `success`, `stopped`, `stopping`.",
)

_VERSIONS = typer.Option(None, help="List of version ids to filter image builds.")

_IMAGE_LIST_NAME = typer.Option(None, help="Image Name or version to search for.")

_IMAGE_LIST_STATUS = typer.Option(
    None,
    help="List of status of the latest version to filter the images by. Valid values are `not-ready`, `ready`, `in-use`, `deleting`, `delete-failed`.",
)

_IMAGE_LIST_SCOPE = typer.Option(
    None,
    help="List of type of image to filter the images by. Valid values are `custom` and `global`.",
)

_IMAGE_LIST_BUILD_STATUS = typer.Option(
    None,
    help="List of build status of the latest version to filter the images by. Valid values are `building`, `failed`, `success`, `stopped`, `stopping`.",
)

_IMAGE_LIST_TAGS = typer.Option(None, help="List of tags on the latest version to filter the images by.")

_VERSION_LIST_VERSION = typer.Option(None, help="Version to search for.")

_VERSION_LIST_STATUS = typer.Option(
    None,
    help="List of statuses to filter the versions by. Valid values are `not-ready`, `ready`, `in-use`, `deleting`, `delete-failed`.",
)

_VERSION_LIST_BUILD_STATUS = typer.Option(
    None,
    help="List of build statuses to filter the versions by. Valid values are `building`, `failed`, `success`, `stopped`, `stopping`.",
)

_VERSION_LIST_TAGS = typer.Option(None, help="List of tags to filter the versions by.")

_NAME = typer.Option(None, help="Name of the image.")

_VERSION = typer.Option(None, help="A valid semantic image version.")

_TYPE = typer.Option(
    None,
    help="Type of the image. Allowed values are 'workflow', 'workspace-r', 'workspace-python, 'api', 'webapp'.",
)

_DESCRIPTION = typer.Option(None, help="Description of the image.")

_ARTIFACT_PATH = typer.Option(None, help="Path to the artifact.")

_ARTIFACT_IGNORE_FILES = typer.Option(None, help="Ignore files to use when creating artifact.")

_BUILD_DETAILS = typer.Option(None, help="Build details of the image. To be passed in stringified json format.")

_SOURCE = typer.Option(
    None,
    help="The source via which the image is to be created. Allowed values are 'github', 'dockerfile' and 'upload'.",
)

_DOCKERFILE = typer.Option(None, help="The dockerfile to be used to create the image.")

_DOCKERFILE_PATH = typer.Option(None, help="Path to the dockerfile.")

_CONTEXT = typer.Option(None, help="The path within the artifact to the code to be executed by the Dockerfile.")

_REPOSITORY = typer.Option(None, help="When source is github, the repository where the Dockerfile content is stored.")

_BRANCH = typer.Option(None, help="When source is github, the Branch that contains the Dockerfile.")

_TOKEN = typer.Option(None, help="When source is github, the token to be used to clone the required repository.")

_USE_CACHE = typer.Option(None, help="Whether to enable image caching to reduce build time.")

_BUILD_ARGUMENTS = typer.Option(None, help="List of build arguments in the format arg1=value1.")

_SECRETS = typer.Option(None, help="List of secret names to be passed as build arguments.")

_VERSIONS_TO_DELETE = typer.Option(..., help="List of version ids to delete.")

MAPPING = {
    "source": "buildDetails",
    "dockerfile": "buildDetails",
    "dockerfilePath": "buildDetails",
    "context": "buildDetails",
    "repository": "buildDetails",
    "branch": "buildDetails",
    "token": "buildDetails",
    "useCache": "buildDetails",
    "buildArguments": "buildDetails",
    "secrets": "buildDetails",  # pragma: allowlist secret
}


@app.command(short_help="Create a new image.")
def create(
    ctx: typer.Context,
    file: Annotated[
        Optional[str],
        typer.Argument(
            ...,
            help="Path to the file that defines the body for this operation, supports both `yaml` file or a `jinja` template.",
        ),
    ] = None,
    params_file: str = args.TEMPLATE_PARAMS_FILE,
    params: List[str] = args.TEMPLATE_PARAMS,
    name: Optional[str] = _NAME,
    version: Optional[str] = _VERSION,
    type: Optional[str] = _TYPE,  # noqa: A002
    description: Optional[str] = _DESCRIPTION,
    artifact_path: Optional[str] = _ARTIFACT_PATH,
    artifact_ignore_files: Optional[List[str]] = _ARTIFACT_IGNORE_FILES,
    build_details: Optional[str] = _BUILD_DETAILS,
    source: Optional[str] = _SOURCE,
    dockerfile: Optional[str] = _DOCKERFILE,
    dockerfile_path: Optional[str] = _DOCKERFILE_PATH,
    context: Optional[str] = _CONTEXT,
    repository: Optional[str] = _REPOSITORY,
    branch: Optional[str] = _BRANCH,
    token: Optional[str] = _TOKEN,
    use_cache: Optional[bool] = _USE_CACHE,
    build_arguments: Optional[List[str]] = _BUILD_ARGUMENTS,
    secrets: Optional[List[str]] = _SECRETS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Create*** a new image. This also adds the first version in the image.

    \b
    🧩 ***Input file schema(yaml):***<br/>
    ```yaml
      body (map):
        name (str): Name of the image.
        version (str | required: false): A valid semantic image version. If not provided, the version will be set to 0.0.1.
        type (str): Type of the image. Allowed values are 'workflow', 'workspace-r', 'workspace-python, 'api' and 'webapp'.
        description (str | required: false): Description of the image.
        buildDetails (map | required: false):
          source (str | required: false): The source via which the image is to be created. Allowed values are 'github', 'dockerfile' and 'upload'. It is 'upload' by default.
          useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
          buildArguments (list(map) | required: false):
            name (str): Name of the build argument.
            value (str): Value of the build argument.
          secrets (list(str)) | required: false): List of secret names to be passed as build arguments.
          context (str | required: false): The path within the artifact to the code to be executed by the Dockerfile.
          dockerfilePath (str | required: false): Path to the Dockerfile inside artifact or the repository.
          repository (str | required: false): When source is github, the repository where the Dockerfile content is stored.
          branch (str | required: false): When source is github, the Branch that contains the Dockerfile.
          token (str | required: false): When source is github, the token to be used to clone the required repository.
          dockerfile (str | required: false): When source is dockerfile, this represents the content of Dockerfile to build the image from.
      artifact (map | required: false):
        path (str): Path to the artifact.
        ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak images create '/path/to/image.yaml' --params-file '/path/to/values.yaml'
    ```

    We can also provide the required parameters from the command line or combine the YAML template and command line arguments in which case the command line arguments will take precedence.

    \b
    📝 ***Example usage without yaml:***
    ```bash
    peak images create --name <name> --type <type> --description <description> --version <version> --source <source> --dockerfile <dockerfile> --secrets <secret_1> --secrets <secret_2>
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "buildId": "build-image-mock:2198c71e-22c5-aca3eac3a55e",
        "imageId": 9999,
        "versionId": 1,
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/images/post_api_v2_images)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    user_options: Dict[str, Any] = variables_to_dict(
        name,
        description,
        type,
        version,
        build_details,
        source,
        dockerfile,
        repository,
        branch,
        token,
        dockerfile_path,
        context,
        use_cache,
        build_arguments,
        secrets,
    )

    user_options = map_user_options(user_options, MAPPING, dict_type_keys=["buildDetails"])

    body: Dict[str, Any] = {}
    if file:
        body = helpers.template_handler(file=file, params_file=params_file, params=params)
        body = helpers.remove_unknown_args(body, image_client.create_image)

    updated_body = combine_dictionaries(body.get("body") or {}, user_options)
    artifact = helpers.get_updated_artifacts(body, artifact_path, artifact_ignore_files)

    if build_arguments:
        updated_body["buildDetails"]["buildArguments"] = helpers.parse_build_arguments(build_arguments)

    if secrets:
        updated_body["buildDetails"]["secrets"] = parse_list_of_strings(secrets)

    with writer.pager():
        response: Dict[str, Any] = image_client.create_image(body=updated_body, artifact=artifact)  # type: ignore  # noqa: PGH003
        writer.write(response)


@app.command(short_help="Create a new image version.")
def create_version(
    ctx: typer.Context,
    image_id: int = _IMAGE_ID,
    file: Annotated[
        Optional[str],
        typer.Argument(
            ...,
            help="Path to the file that defines the body for this operation, supports both `yaml` file or a `jinja` template.",
        ),
    ] = None,
    params_file: str = args.TEMPLATE_PARAMS_FILE,
    params: List[str] = args.TEMPLATE_PARAMS,
    version: Optional[str] = _VERSION,
    description: Optional[str] = _DESCRIPTION,
    artifact_path: Optional[str] = _ARTIFACT_PATH,
    artifact_ignore_files: Optional[List[str]] = _ARTIFACT_IGNORE_FILES,
    build_details: Optional[str] = _BUILD_DETAILS,
    source: Optional[str] = _SOURCE,
    dockerfile: Optional[str] = _DOCKERFILE,
    dockerfile_path: Optional[str] = _DOCKERFILE_PATH,
    context: Optional[str] = _CONTEXT,
    repository: Optional[str] = _REPOSITORY,
    branch: Optional[str] = _BRANCH,
    token: Optional[str] = _TOKEN,
    use_cache: Optional[bool] = _USE_CACHE,
    build_arguments: Optional[List[str]] = _BUILD_ARGUMENTS,
    secrets: Optional[List[str]] = _SECRETS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Create*** a new version in an existing image.

    \b
    🧩 ***Input file schema(yaml):***<br/>
    ```yaml
      body (map):
        version (str | required: false): A valid semantic image version. If not provided, the next patch version of the latest version will be used.
        description (str | required: false): Description of the image version.
        buildDetails (map | required: false):
          source (str | required: false): The source via which the image version is to be created. Allowed values are 'github', 'dockerfile' and 'upload'. It is 'upload' by default.
          useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
          buildArguments (list(map) | required: false):
            name (str): Name of the build argument.
            value (str): Value of the build argument.
          secrets (list(str) | required: false): List of secret names to be passed as build arguments.
          context (str | required: false): The path within the artifact to code to be executed by the Dockerfile.
          dockerfilePath (str | required: false): Path to the Dockerfile inside artifact or repository.
          repository (str | required: false): When source is github, the repository where the Dockerfile content is stored.
          branch (str | required: false): When source is github, the Branch that contains the Dockerfile.
          token (str | required: false): When source is github, the token to be used to clone the required repository.
          dockerfile (str | required: false): When source is dockerfile, this represents the content of Dockerfile to build the image from.
        autodeployResources (list(map) | required: false): A list of resources that should be redeployed when the build completes.
            entityId (str): The id of the resource to be redeployed.
            entityType (str): The type of the resource to be redeployed. Allowed values are 'workflow', 'workspace', 'api' and 'webapp'.
      artifact (map | required: false):
        path (str): Path to the artifact.
        ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak images create-version <imageId> '/path/to/file.yaml' -v '/path/to/parmas.yaml'
    ```

    We can also provide the required parameters from the command line or combine the YAML template and command line arguments in which case the command line arguments will take precedence.

    \b
    📝 ***Example usage without yaml:***
    ```bash
    peak images create-version <image_id> --description <description> --version <version> --source <source> --dockerfile <dockerfile> --secrets <secret_1>,<secret_2>
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "buildId": "build-image-mock:2198c71e-22c5-aca3eac3a55e",
        "versionId": 101,
        "autodeploymentId": "0a12abcd-11ab-22d2-123a-a1234b333abc"
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/post_api_v2_images__imageId__versions)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    user_options: Dict[str, Any] = variables_to_dict(
        description,
        version,
        build_details,
        source,
        dockerfile,
        repository,
        branch,
        token,
        dockerfile_path,
        context,
        use_cache,
        build_arguments,
        secrets,
    )

    user_options = map_user_options(user_options, MAPPING, dict_type_keys=["buildDetails"])

    body: Dict[str, Any] = {}
    if file:
        body = helpers.template_handler(file=file, params_file=params_file, params=params)
        body = helpers.remove_unknown_args(body, image_client.create_version)

    updated_body = combine_dictionaries(
        body.get("body") or {},
        user_options,
    )
    artifact = helpers.get_updated_artifacts(body, artifact_path, artifact_ignore_files)

    if build_arguments:
        updated_body["buildDetails"]["buildArguments"] = helpers.parse_build_arguments(build_arguments)

    if secrets:
        updated_body["buildDetails"]["secrets"] = parse_list_of_strings(secrets)

    with writer.pager():
        response: Dict[str, Any] = image_client.create_version(image_id=image_id, body=updated_body, artifact=artifact)  # type: ignore  # noqa: PGH003
        writer.write(response)


@app.command(
    short_help="Create a new image/version or Update and existing version.",
)
def create_or_update(
    ctx: typer.Context,
    file: Annotated[
        Optional[str],
        typer.Argument(
            ...,
            help="Path to the file that defines the body for this operation, supports both `yaml` file or a `jinja` template.",
        ),
    ] = None,
    params_file: str = args.TEMPLATE_PARAMS_FILE,
    params: List[str] = args.TEMPLATE_PARAMS,
    name: Optional[str] = _NAME,
    version: Optional[str] = _VERSION,
    type: Optional[str] = _TYPE,  # noqa: A002
    description: Optional[str] = _DESCRIPTION,
    artifact_path: Optional[str] = _ARTIFACT_PATH,
    artifact_ignore_files: Optional[List[str]] = _ARTIFACT_IGNORE_FILES,
    build_details: Optional[str] = _BUILD_DETAILS,
    source: Optional[str] = _SOURCE,
    dockerfile: Optional[str] = _DOCKERFILE,
    dockerfile_path: Optional[str] = _DOCKERFILE_PATH,
    context: Optional[str] = _CONTEXT,
    repository: Optional[str] = _REPOSITORY,
    branch: Optional[str] = _BRANCH,
    token: Optional[str] = _TOKEN,
    use_cache: Optional[bool] = _USE_CACHE,
    build_arguments: Optional[List[str]] = _BUILD_ARGUMENTS,
    secrets: Optional[List[str]] = _SECRETS,
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Create*** a new image if it doesn't exist. This also adds the first version in the image. In case image exists, it will add a new version to the image. ***Update*** a version if exists.

    \b
    🧩 ***Input file schema(yaml):***<br/>
    ```yaml
      body (map):
        name (str): Name of the image.
        version (str | required: false): A valid semantic image version. If not provided, the next patch version of the latest version will be used.
        type (str): Type of the image. Allowed values are 'workflow', 'workspace-r', 'workspace-python, 'api' and 'webapp'.
        description (str | required: false): Description of the image.
        buildDetails (map | required: false): If not provided and if the image version already exists, the build details of the existing version will be used.
          source (str | required: false): The source via which the image is to be created. Allowed values are 'github', 'dockerfile' and 'upload'. It is 'upload' by default if new version is being created otherwise the existing version's source will be used.
          useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
          buildArguments (list(map) | required: false):
            name (str): Name of the build argument.
            value (str): Value of the build argument.
          secrets (list(str)) | required: false): List of secret names to be passed as build arguments.
          context (str | required: false): The path within the artifact to the code to be executed by the Dockerfile.
          dockerfilePath (str | required: false): Path to the Dockerfile inside artifact or the repository.
          repository (str | required: false): When source is github, the repository where the Dockerfile content is stored.
          branch (str | required: false): When source is github, the Branch that contains the Dockerfile.
          token (str | required: false): When source is github, the token to be used to clone the required repository.
          dockerfile (str | required: false): When source is dockerfile, this represents the content of Dockerfile to build the image from.
      artifact (map | required: false):
        path (str): Path to the artifact.
        ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak images create-or-update '/path/to/image.yaml' --params-file '/path/to/values.yaml'
    ```

    \b
    We can also provide the required parameters from the command line or combine the YAML template and command line arguments in which case the command line arguments will take precedence.

    \b
    📝 ***Example usage without yaml:***
    ```bash
    peak images create-or-update --name <name> --type <type> --description <description> --version <version> --source <source> --dockerfile <dockerfile> --secrets <secret_1> --secrets <secret_2>
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "buildId": "build-image-mock:2198c71e-22c5-aca3eac3a55e",
        "imageId": 9999,
        "versionId": 1,
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/images/post_api_v2_images)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    user_options: Dict[str, Any] = variables_to_dict(
        name,
        description,
        type,
        version,
        build_details,
        source,
        dockerfile,
        repository,
        branch,
        token,
        dockerfile_path,
        context,
        use_cache,
        build_arguments,
        secrets,
    )

    user_options = map_user_options(user_options, MAPPING, dict_type_keys=["buildDetails"])

    body: Dict[str, Any] = {}
    if file:
        body = helpers.template_handler(file=file, params_file=params_file, params=params)
        body = helpers.remove_unknown_args(body, image_client.create_or_update_image_version)

    updated_body = combine_dictionaries(body.get("body") or {}, user_options)
    artifact = helpers.get_updated_artifacts(body, artifact_path, artifact_ignore_files)

    if build_arguments:
        updated_body["buildDetails"]["buildArguments"] = helpers.parse_build_arguments(build_arguments)

    if secrets:
        updated_body["buildDetails"]["secrets"] = parse_list_of_strings(secrets)

    with writer.pager():
        response: Dict[str, Any] = image_client.create_or_update_image_version(body=updated_body, artifact=artifact)  # type: ignore  # noqa: PGH003
        writer.write(response)


@app.command("list", short_help="List images.")
def list_images(
    ctx: typer.Context,
    page_size: Optional[int] = args.PAGE_SIZE,
    page_number: Optional[int] = args.PAGE_NUMBER,
    name: Optional[str] = _IMAGE_LIST_NAME,
    status: Optional[List[str]] = _IMAGE_LIST_STATUS,
    scope: Optional[List[str]] = _IMAGE_LIST_SCOPE,
    last_build_status: Optional[List[str]] = _IMAGE_LIST_BUILD_STATUS,
    tags: Optional[List[str]] = _IMAGE_LIST_TAGS,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** all images that exists in the tenant along with details about their latest version.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images list --page-size 10 --page-number 1 --name "test" --status "ready,in-use" --scope "custom" --last-build-status "building" --tags "tag1"
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "imageCount": 1,
        "images": [...],
        "pageCount": 1,
        "pageNumber": 1,
        "pageSize": 25
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/images/get_api_v2_images)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = image_client.list_images(
            page_size=page_size,
            page_number=page_number,
            return_iterator=False,
            name=name,
            status=parse_list_of_strings(status),
            scope=parse_list_of_strings(scope),
            last_build_status=parse_list_of_strings(last_build_status),
            tags=parse_list_of_strings(tags),
        )

        writer.write(response)


@app.command("list-versions", short_help="List image versions.")
def list_image_versions(
    ctx: typer.Context,
    image_id: int = _IMAGE_ID,
    page_size: Optional[int] = args.PAGE_SIZE,
    page_number: Optional[int] = args.PAGE_NUMBER,
    version: Optional[str] = _VERSION_LIST_VERSION,
    status: Optional[List[str]] = _VERSION_LIST_STATUS,
    last_build_status: Optional[List[str]] = _VERSION_LIST_BUILD_STATUS,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    tags: Optional[List[str]] = _VERSION_LIST_TAGS,
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** all versions for an image.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images list-versions 9999 --page-size 10 --page-number 1 --version "0.0.1" --status "ready,in-use" --last-build-status "failed"  --tags "tag1,tag2"
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "versionCount": 1,
        "versions": [...],
        "pageCount": 1,
        "pageNumber": 1,
        "pageSize": 25
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/get_api_v2_images__imageId__versions)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response: Dict[str, Any] = image_client.list_image_versions(
            image_id=image_id,
            page_size=page_size,
            page_number=page_number,
            return_iterator=False,
            version=version,
            status=parse_list_of_strings(status),
            last_build_status=parse_list_of_strings(last_build_status),
            tags=parse_list_of_strings(tags),
        )

        writer.write(response)


@app.command(short_help="Describe details of a specific image.")
def describe(
    ctx: typer.Context,
    image_id: int = _IMAGE_ID,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Describe*** details of a specific image and its latest version.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images describe 9999
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "id": 1,
        "name": "awesome-image",
        "type": "workflow",
        "scope": "custom",
        "createdAt": "2020-01-01T18:00:00.000Z",
        "createdBy": "someone@peak.ai",
        "latestVersion": {...}
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/images/get_api_v2_images__imageId_)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = image_client.describe_image(image_id=image_id)
        writer.write(response)


@app.command(short_help="Describe details of a specific version.")
def describe_version(
    ctx: typer.Context,
    image_id: int = args.IMAGE_ID,
    version_id: int = args.VERSION_ID,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Describe*** details of a specific version.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images describe-version --image-id 9999 --version-id 101
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "imageName": "awesome-image",
        "imageType": "Workflow",
        "id": 101,
        "version": "0.0.1-python3.9",
        "description": "This is an awesome image",
        "status": "not-ready",
        "createdAt": "2020-01-01T18:00:00.000Z",
        "createdBy": "someone@peak.ai",
        "updatedAt": "2020-01-01T18:00:00.000Z",
        "updatedBy": "someone@peak.ai",
        "pullUrl": "<image-url>",
        "lastBuildStatus": "building",
        "lastBuildAt": "2020-01-01T18:00:00.000Z",
        "buildDetails": {...},
        "tags": [...]
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/get_api_v2_images__imageId__versions__versionId_)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response: Dict[str, Any] = image_client.describe_version(image_id=image_id, version_id=version_id)
        writer.write(response)


@app.command(short_help="Update an image version.")
def update_version(
    ctx: typer.Context,
    image_id: int = args.IMAGE_ID,
    version_id: int = args.VERSION_ID,
    file: Annotated[
        Optional[str],
        typer.Argument(
            ...,
            help="Path to the file that defines the body for this operation, supports both `yaml` file or a `jinja` template.",
        ),
    ] = None,
    params_file: str = args.TEMPLATE_PARAMS_FILE,
    params: List[str] = args.TEMPLATE_PARAMS,
    description: Optional[str] = _DESCRIPTION,
    artifact_path: Optional[str] = _ARTIFACT_PATH,
    artifact_ignore_files: Optional[List[str]] = _ARTIFACT_IGNORE_FILES,
    build_details: Optional[str] = _BUILD_DETAILS,
    source: Optional[str] = _SOURCE,
    dockerfile: Optional[str] = _DOCKERFILE,
    dockerfile_path: Optional[str] = _DOCKERFILE_PATH,
    context: Optional[str] = _CONTEXT,
    repository: Optional[str] = _REPOSITORY,
    branch: Optional[str] = _BRANCH,
    token: Optional[str] = _TOKEN,
    use_cache: Optional[bool] = _USE_CACHE,
    build_arguments: Optional[List[str]] = _BUILD_ARGUMENTS,
    secrets: Optional[List[str]] = _SECRETS,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
    generate: Optional[bool] = GENERATE_YAML,  # noqa: ARG001
) -> None:
    """***Update*** an image version. Only the versions that are in `not-ready` state can be updated and you can not update their version while updating them.

    \b
    🧩 ***Input file schema(yaml):***<br/>
    ```yaml
    body (map):
      description (str | required: false): Description of the image version.
      buildDetails (map | required: false): If not provided, the existing build details will be used.
        source (str | required: false): The source via which the image version is to be created. Allowed values are 'github', 'dockerfile' and 'upload'. Source of existing version will be used if not provided.
        useCache (boolean | required: false): Whether to enable image caching to reduce build time, is enabled by default.
        buildArguments (list(map) | required: false):
          name (str): Name of the build argument.
          value (str): Value of the build argument.
        secrets (list(str) | required: false): List of secret names to be passed as build arguments.
        context (str | required: false): The path within the artifact to code to be executed by the Dockerfile.
        dockerfilePath (str | required: false): Path to the Dockerfile inside artifact or repository.
        repository (str | required: false): When source is github, the repository where the Dockerfile content is stored.
        branch (str | required: false): When source is github, the Branch that contains the Dockerfile.
        token (str | required: false): When source is github, the token to be used to clone the required repository.
        dockerfile (str | required: false): When source is dockerfile, this represents the content of Dockerfile to build the image from.
      autodeployResources (list(map) | required: false): A list of resources that should be redeployed when the build completes.
            entityId (str): The id of the resource to be redeployed.
            entityType (str): The type of the resource to be redeployed. Allowed values are 'workflow', 'workspace', 'api' and 'webapp'.
    artifact (map | required: false):
      path (str): Path to the artifact.
      ignore_files (list(str) | required: false) : Ignore files to use when creating artifact.
    ```

    \b
    📝 ***Example usage:***
    ```bash
    peak images update-version --image-id 9999 --version-id 101 '/path/to/file.yaml' -v '/path/to/parmas.yaml'
    ```

    We can also provide the required parameters from the command line or combine the YAML template and command line arguments in which case the command line arguments will take precedence.

    \b
    📝 ***Example usage without yaml:***
    ```bash
    peak images update-version --image-id <image_id> --version-id <version_id> --description <description> --source <source> --dockerfile <dockerfile> --secrets <secret_1>,<secret_2>
    ```

    \b
    🆗 ***Response:***
    ```json
    {
        "buildId": "build-image-mock:2198c71e-22c5-aca3eac3a55e",
        "versionId": 101,
        "autodeploymentId": "0a12abcd-11ab-22d2-123a-a1234b333abc"
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/patch_api_v2_images__imageId__versions__versionId_)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    user_options: Dict[str, Any] = variables_to_dict(
        description,
        build_details,
        source,
        dockerfile,
        repository,
        branch,
        token,
        dockerfile_path,
        context,
        use_cache,
        build_arguments,
        secrets,
    )

    user_options = map_user_options(user_options, MAPPING, dict_type_keys=["buildDetails"])

    body: Dict[str, Any] = {}
    if file:
        body = helpers.template_handler(file=file, params_file=params_file, params=params)
        body = helpers.remove_unknown_args(body, image_client.update_version)

    updated_body = combine_dictionaries(
        body.get("body") or {},
        user_options,
    )
    artifact = helpers.get_updated_artifacts(body, artifact_path, artifact_ignore_files)

    if build_arguments:
        updated_body["buildDetails"]["buildArguments"] = helpers.parse_build_arguments(build_arguments)

    if secrets:
        updated_body["buildDetails"]["secrets"] = parse_list_of_strings(secrets)

    with writer.pager():
        response: Dict[str, str] = image_client.update_version(
            image_id=image_id,
            version_id=version_id,
            body=updated_body,
            artifact=artifact,  # type: ignore  # noqa: PGH003
        )
        writer.write(response)


@app.command(short_help="Delete an image.")
def delete(
    ctx: typer.Context,
    image_id: int = _IMAGE_ID,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Delete*** an image. All the versions in the image will first go into `deleting` state before actually being deleted.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images delete 9999
    ```

    \b
    🆗 ***Response:***
    ```json
    {}
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/images/delete_api_v2_images__imageId_)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = image_client.delete_image(image_id=image_id)
        writer.write(response)


@app.command(short_help="Delete an image version.")
def delete_version(
    ctx: typer.Context,
    image_id: int = args.IMAGE_ID,
    version_id: int = args.VERSION_ID,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Delete*** an image version. An image cannot be deleted if it is being used by any other resource.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images delete-version --image-id 9999 --version-id 1
    ```

    \b
    🆗 ***Response:***
    ```json
    {}
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/delete_api_v2_images__imageId__versions__versionId_)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response: Dict[None, None] = image_client.delete_version(image_id=image_id, version_id=version_id)
        writer.write(response)


@app.command(short_help="Delete all the specified versions.")
def delete_versions(
    ctx: typer.Context,
    image_id: int = args.IMAGE_ID,
    version_ids: List[str] = _VERSIONS_TO_DELETE,
    dry_run: Optional[bool] = DRY_RUN,  # noqa: ARG001
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Delete*** all the specified versions for an image.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images delete-versions --image-id 9999 --version-ids 1 --version-ids 2
    ```

    \b
    🆗 ***Response:***
    ```json
    {}
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/versions/delete_api_v2_images__imageId__versions)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]
    parsed_version_ids = [int(version_id) for version_id in parse_list_of_strings(version_ids) or []]

    with writer.pager():
        response: Dict[None, None] = image_client.delete_versions(
            image_id=image_id,
            version_ids=parsed_version_ids,
        )
        writer.write(response)


@app.command(short_help="List image builds.")
def list_builds(
    ctx: typer.Context,
    image_id: int = _IMAGE_ID,
    count: Optional[int] = _COUNT,
    version_ids: Optional[List[str]] = _VERSIONS,
    build_status: Optional[List[str]] = _BUILD_STATUS,
    date_from: Optional[str] = args.DATE_FROM,
    date_to: Optional[str] = args.DATE_TO,
    page_size: Optional[int] = args.PAGE_SIZE,
    page_number: Optional[int] = args.PAGE_NUMBER,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** image builds for a specific image. If you want to view builds for a specific version you can pass in Version's ID to the command.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images list-builds 9999 --version-ids 1,2,3 --page-size 10 --page-number 1
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "buildCount": 1,
        "builds": [...],
        "pageCount": 1,
        "pageNumber": 1,
        "pageSize": 25
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/builds/get_api_v2_images__imageId__builds)
    """
    image_client: Image = ctx.obj["client"]
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = image_client.list_image_builds(
            image_id=image_id,
            count=count,
            build_status=parse_list_of_strings(build_status),
            version_ids=parse_list_of_strings(version_ids),
            date_from=date_from,
            date_to=date_to,
            page_size=page_size,
            page_number=page_number,
            return_iterator=False,
        )

        writer.write(response)


@app.command(short_help="Get image build logs.")
def get_build_logs(
    ctx: typer.Context,
    image_id: int = args.IMAGE_ID,
    build_id: int = _BUILD_ID,
    next_token: Optional[str] = args.NEXT_TOKEN,
    follow: Optional[bool] = args.FOLLOW,
    save: Optional[bool] = args.SAVE,
    file_name: Optional[str] = args.FILE_NAME,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesOnlyJson] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Get*** logs for an image build.

    \b
    If you want to save the logs to a file, you can use the `--save` flag.
    If you don't provide a file name, the logs will be saved to a file with the name `image_build_logs_{image_id}_{build_id}.log`.

    \b
    If you want to view next set of logs you can pass in the `nextToken` to the command.

    \b
    📝 ***Example usage:***<br/>
    ```bash
    peak images get-build-logs --image-id <image_id> --build-id <build_id> --next-token <next_token>
    ```

    \b
    To follow the logs, you can use the `--follow` flag.

    \b
    📝 ***Example usage to follow the logs:***<br/>
    ```bash
    peak images get-build-logs --image-id <image_id> --build-id <build_id> --follow
    ```

    \b
    🆗 ***Response:***
    ```
    {
        "buildStatus": "building",
        "finishTime": "2023-06-01T19:00:00.000Z",
        "logs": [
            {
                "ingestionTime": "2023-06-01T18:00:00.000Z",
                "message": "Building the Docker image...",
                "timestamp": "2023-06-01T18:00:00.000Z"
            }
        ],
        "nextToken": "f/37241814580157116960215105647056337845181039459911335941/s",
        "startTime": "2023-06-01T18:00:00.000Z"
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/image-management/api-docs/index.htm#/builds/get_api_v2_images__imageId__builds__buildId__logs)
    """
    image_client: Image = ctx.obj["client"]
    writer = Writer()

    pager = writer.pager()  # Create pager context

    with pager:
        while True:
            response = image_client.get_build_logs(
                image_id=image_id,
                build_id=build_id,
                next_token=next_token,
                save=save,
                file_name=file_name,
            )

            if save or not response or "buildStatus" not in response or "logs" not in response:
                break

            if follow:
                if (response["buildStatus"] != "building") and (response["logs"] is None or len(response["logs"]) == 0):
                    break
                formatted_logs = helpers.format_logs(response["logs"])
                if len(formatted_logs):
                    writer.write(formatted_logs)
            else:
                writer.write(response)
                break

            next_token = response.get("nextToken", None)



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#

"""Webapp client module.

.. warning::
    The ``Webapp`` module is deprecated and will eventually be removed. It is recommended that users begin transitioning to the ``Service`` module, which offers all the features of ``Webapp`` along with the added capability of API deployment and other significant updates for enhanced functionality. Transitioning early to the ``Service`` module will ensure easier adoption of its advanced features and future updates.

    For guidance on migrating to the ``Service`` module and taking full advantage of its capabilities, please refer to the ``Service`` module's documentation.
"""
from __future__ import annotations

from typing import Any, Dict, Iterator, List, Literal, Optional, overload

from peak.base_client import BaseClient
from peak.constants import ContentType, HttpMethods
from peak.session import Session


class Webapp(BaseClient):
    """Client class for interacting with webapps resource."""

    BASE_ENDPOINT = "webapps/api/v1"

    @overload
    def list_webapps(
        self,
        page_size: Optional[int] = None,
        page_number: Optional[int] = None,
        status: Optional[List[str]] = None,
        name: Optional[str] = None,
        *,
        return_iterator: Literal[False],
    ) -> Dict[str, Any]: ...

    @overload
    def list_webapps(
        self,
        page_size: Optional[int] = None,
        page_number: Optional[int] = None,
        status: Optional[List[str]] = None,
        name: Optional[str] = None,
        *,
        return_iterator: Literal[True] = True,
    ) -> Iterator[Dict[str, Any]]: ...

    def list_webapps(
        self,
        page_size: Optional[int] = None,
        page_number: Optional[int] = None,
        status: Optional[List[str]] = None,
        name: Optional[str] = None,
        *,
        return_iterator: bool = True,
    ) -> Iterator[Dict[str, Any]] | Dict[str, Any]:
        """Retrieve a list of webapps.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/list-service>`__

        Args:
            page_size (int | None): The number of webapps per page.
            page_number (int | None): The page number to retrieve. Only used when `return_iterator` is False.
            status (List[str] | None): A list of webapp status to filter the list by.
                Valid values are `CREATING`, `DEPLOYING`, `AVAILABLE`, `DELETING`, `CREATE_FAILED`, `DELETE_FAILED`.
            name (str | None): Name of the webapp to search for.
            return_iterator (bool): Whether to return an iterator object or list of webapps for a specified page number, defaults to True.

        Returns:
            Iterator[Dict[str, Any] | Dict[str, Any]: an iterator object which returns an element per iteration, until there are no more elements to return.
            If `return_iterator` is set to False, a dictionary containing the list and pagination details is returned instead.
            Set `return_iterator` to True if you want automatic client-side pagination, or False if you want server-side pagination.

        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.GET, f"{self.BASE_ENDPOINT}/webapps/"
        params = {"pageSize": page_size, "status": status, "searchTerm": name}

        if return_iterator:
            return self.session.create_generator_request(
                endpoint,
                method,
                content_type=ContentType.APPLICATION_JSON,
                response_key="webapps",
                params=params,
            )

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
            params={**params, "pageNumber": page_number},
        )

    def create_webapp(self, body: Dict[str, Any]) -> Dict[str, str]:
        """Create a new webapp. Only geneic (EKS-based) webapps are supported at the moment.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/create-service>`__

        Args:
            body (Dict[str, Any]): A dictionary containing the webapp config. Schema can be found below.

        Returns:
            Dict[str, str]: Id of the new webapp

        SCHEMA:
            .. code-block:: json

                {
                    "name": "string(required)",
                    "title": "string",
                    "imageDetails": {
                        "imageId": "number(required)",
                        "versionId": "number",
                    },
                    "resources": {
                        "instanceTypeId": "number"
                    },
                    "description": "string",
                    "sessionStickiness": "boolean"
                }

        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.POST, f"{self.BASE_ENDPOINT}/webapps/"

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
            body=body,
        )

    def update_webapp(
        self,
        webapp_id: str,
        body: Dict[str, Any],
    ) -> Dict[str, str]:
        """Updates the existing webapp. Only geneic (EKS-based) webapps are supported at the moment.

        When updating the webapp, it will trigger a redeployment only under specific conditions.
        Redeployment is triggered if you make changes to any of the following parameters: imageId, versionId, instanceTypeId or sessionStickiness.
        However, only modifying the title or description will not trigger a redeployment.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/update-service>`__

        Args:
            webapp_id (str): The ID of the webapp to update.
            body (Dict[str, Any]): A dictionary containing the webapp config. Schema can be found below.

        Returns:
            Dict[str, str]: Id of the webapp.

        SCHEMA:
            .. code-block:: json

                {
                    "title": "string",
                    "imageDetails": {
                        "imageId": "number(required)",
                        "versionId": "number",
                    },
                    "resources": {
                        "instanceTypeId": "number"
                    },
                    "description": "string",
                    "sessionStickiness": "boolean"
                }

        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            NotFoundException: The given webapp does not exist.
            UnprocessableEntityException: The server was unable to process the request.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.PATCH, f"{self.BASE_ENDPOINT}/webapps/{webapp_id}"
        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
            body=body,
        )

    def create_or_update_webapp(self, body: Dict[str, Any]) -> Dict[str, str]:
        """Create a new webapp or updates an existing webapp based on webapp name.

        When updating the webapp, it will trigger a redeployment only under specific conditions.
        Redeployment is triggered if you make changes to any of the following parameters: imageId, versionId, instanceTypeId or sessionStickiness.
        However, only modifying the title or description will not trigger a redeployment.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/create-service>`__

        Args:
            body (Dict[str, Any]): A dictionary containing the webapp config. Schema can be found below.

        Returns:
            Dict[str, str]: Id of the new or updated webapp.

        SCHEMA:
            .. code-block:: json

                {
                    "name": "string(required)",
                    "title": "string",
                    "imageDetails": {
                        "imageId": "number(required)",
                        "versionId": "number"
                    },
                    "resources": {
                        "instanceTypeId": "number"
                    },
                    "description": "string",
                    "sessionStickiness": "boolean"
                }


        Raises:
            BadRequestException: The given request parameters are invalid.
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            InternalServerErrorException: The server failed to process the request.
        """
        webapp_name = body.get("name", "")
        response = (
            {} if not len(webapp_name) else self.list_webapps(page_size=100, return_iterator=False, name=webapp_name)
        )
        filtered_webapps = list(
            filter(lambda webapp: webapp.get("name", "") == webapp_name, response.get("webapps", [])),
        )

        if len(filtered_webapps) > 0:
            webapp_id = filtered_webapps[0]["id"]
            return self.update_webapp(webapp_id=webapp_id, body=body)

        return self.create_webapp(body=body)

    def delete_webapp(
        self,
        webapp_id: str,
    ) -> Dict[str, str]:
        """Delete a webapp.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/delete-service>`__

        Args:
            webapp_id (str): The ID of the webapp to delete.

        Returns:
            Dict[str, str]: Dictonary containing Id of the deleted webapp

        Raises:
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            NotFoundException: The given webapp does not exist.
            ConflictException: If the webapp is in a conflicting state while deleting.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.DELETE, f"{self.BASE_ENDPOINT}/webapps/{webapp_id}"

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
        )

    def describe_webapp(
        self,
        webapp_id: str,
    ) -> Dict[str, Any]:
        """Retrieve details of a specific webapp.

        REFERENCE:
            🔗 `API Documentation <https://service.peak.ai/webapps/api-docs/index.htm#/Services/get-service>`__

        Args:
            webapp_id (str): The ID of the webapp to fetch.

        Returns:
            Dict[str, Any]: Dictonary containing details of the webapp

        Raises:
            UnauthorizedException: The credentials are invalid.
            ForbiddenException: The user does not have permission to perform the operation.
            NotFoundException: The given webapp does not exist.
            UnprocessableEntityException: The server was unable to process the request.
            InternalServerErrorException: The server failed to process the request.
        """
        method, endpoint = HttpMethods.GET, f"{self.BASE_ENDPOINT}/webapps/{webapp_id}"

        return self.session.create_request(  # type: ignore[no-any-return]
            endpoint,
            method,
            content_type=ContentType.APPLICATION_JSON,
        )


def get_client(session: Optional[Session] = None) -> Webapp:
    """Returns a Webapp client, If no session is provided, a default session is used.

    Args:
        session (Optional[Session]): A Session Object. Default is None.

    Returns:
        Webapp: the webapp client object
    """
    return Webapp(session)


__all__ = ["get_client"]



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Peak Users commands."""
from typing import Optional

import typer
from peak import Session
from peak.cli.args import OUTPUT_TYPES, PAGING
from peak.constants import OutputTypes, OutputTypesNoTable
from peak.output import Writer
from peak.resources import users

app = typer.Typer(
    help="User management and permission checking.",
    short_help="Manage User Permissions.",
)

_FEATURE = typer.Option(..., help="The feature path to check permissions for, e.g., 'PRICING.GUARDRAILS'.")
_ACTION = typer.Option(..., help="The action to check for the feature path, e.g., 'read' or 'write'.")
_AUTH_TOKEN = typer.Option(
    None,
    help="Authentication token for the user. If not provided, the token from the environment will be used.",
)


@app.command(short_help="Check user permission for a specific feature.")
def check_permission(
    ctx: typer.Context,
    feature: str = _FEATURE,
    action: str = _ACTION,
    auth_token: Optional[str] = _AUTH_TOKEN,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """Check if the user has the specified permission for the given feature path.

    \b
    📝 ***Example usage:***
    ```bash
    peak user check-permission --feature "PRICING.GUARDRAILS" --action "read" --auth-token <your-auth-token>
    ```

    \b
    🆗 ***Response:***
    True if the user has the permission, False otherwise.
    """
    user_session = Session(auth_token=auth_token)
    user_client = users.get_client(session=user_session)
    writer: Writer = ctx.obj["writer"]

    with writer.pager():
        response = user_client.check_permissions({feature: action})
        writer.write(response.get(feature, False), output_type=OutputTypes.json)



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""CLI arguments."""

import typer
from peak.callbacks import dry_run, generate_yaml, handle_output, paging
from peak.cli.version import display_version
from peak.constants import OutputTypes

VERSION_OPTION = typer.Option(False, "--version", callback=display_version, is_eager=True)

TEMPLATE_PATH = typer.Argument(
    ...,
    help="""
        Path to the file that defines the body for this operation, supports both `yaml` file or a `jinja` template.
    """,
)

TEMPLATE_DESCRIPTION_FILE = typer.Option(
    None,
    "--desc-file",
    "-d",
    help="""
        Path to the file that defines the description for this operation, supports `md`, and `txt` files only.
    """,
)

TEMPLATE_PARAMS_FILE = typer.Option(
    None,
    "--params-file",
    "-v",
    help="Path to the `yaml` file containing the parameter values map for the template file.",
)

TEMPLATE_PARAMS = typer.Option(
    None,
    "--params",
    "-p",
    help="Parameters to be used with the template file. Overrides the parameters in the params file. Parameters should be in the format `key=value`.",
)

PAGE_NUMBER = typer.Option(None, help="The page number to retrieve.")

PAGE_SIZE = typer.Option(None, help="Number of entities to include per page.")

DATE_FROM = typer.Option(None, help="The date after which the entities should be included (in ISO format).")

DATE_TO = typer.Option(None, help="The date till which the entities should be included (in ISO format).")

IMAGE_ID = typer.Option(..., help="ID of the image to be used in this operation.")

VERSION_ID = typer.Option(..., help="ID of the version to be used in this operation.")

SPEC_ID = typer.Option(..., help="ID of the spec to be used in this operation.")

DEPLOYMENT_ID = typer.Option(..., help="ID of the deployment to be used in this operation.")

RELEASE_VERSION = typer.Option(..., help="Release version to be used in this operation.")

REVISION = typer.Option(..., help="Revision number of the Block deployment revision to be used in this operation")

SORT_KEYS = typer.Option(
    None,
    help="A comma-separated list of fields to order results by, in the format \\<field\\>:\\<order\\>.",
)

STATUS_FILTER_SPECS = typer.Option(
    None,
    help="A comma-separated list of statuses to filter specs. Valid values are `available`, `unavailable` and `archived`.",
)

STATUS_FILTER_SPEC_RELEASES = typer.Option(
    None,
    help="A comma-separated list of statuses to filter spec releases. Valid values are `deleting`, `delete_failed`, `deployed`, `deploying`, `failed`, `platform_resource_error`, `redeploying`, `rollback`, `rollback_complete`, `rollback_failed` and `warning`.",
)

STATUS_FILTER_DEPLOYMENTS = typer.Option(
    None,
    help="A comma-separated list of statuses to filter deployments. Valid values are `deploying`, `deployed`, `deleting`, `delete_failed`, `failed`, `platform_resource_error`, `redeploying`, `rollback`, `rollback_complete`, `rollback_failed`, and `warning`.",
)

STATUS_FILTER_DEPLOYMENT_REVISIONS = typer.Option(
    None,
    help="A comma-separated list of statuses to filter deployments. Valid values are `deleting`, `delete_failed`, `deployed`, `deploying`, `failed`, `platform_resource_error`, `rollback`, `rollback_complete`, `rollback_failed` and `superseded`",
)


NAME_FILTER = typer.Option(None, help="Only return entities whose names begins with the query string.")

TITLE_FILTER = typer.Option(None, help="Only return entities whose title begins with the query string.")

SCOPES = typer.Option(None, help="A comma-separated list of scopes to only return entities of those scopes.")

FEATURED = typer.Option(None, "--featured", help="Whether to only return featured entities.")

KIND_FILTER = typer.Option(None, help="Only return entities with the kind specified.")

TERM_FILTER = typer.Option(
    None,
    help="Only return entities which contain the term in name, title, description or summary.",
)

DRY_RUN = typer.Option(
    False,
    "--dry-run",
    help="If set, prints the debug information instead of sending the actual request.",
    callback=dry_run,
)

PAGING = typer.Option(
    False,
    "--paging",
    help="If set, prints the output using the default pager.",
    callback=paging,
)

OUTPUT_TYPES = typer.Option(
    OutputTypes.json,
    "--output",
    "-o",
    help="The output format to display data in.",
    callback=handle_output,
)

GENERATE_YAML = typer.Option(
    False,
    "--generate",
    help="Generate Sample YAML that can be used in this operation.",
    callback=generate_yaml,
)

NEXT_TOKEN = typer.Option(None, help="The token to retrieve the next set of logs.")

FOLLOW = typer.Option(False, help="Whether to follow the logs.")

SAVE = typer.Option(
    False,
    help="If set, saves the logs locally to log file.",
)

FILE_NAME = typer.Option(
    None,
    help="Name or path of the file to save the logs.",
)

RELEASE_NOTES_FILE = typer.Option(
    None,
    "--release-notes-file",
    "-n",
    help="""
        Path to the file that defines the release notes for this operation, supports `md`, and `txt` files only.
    """,
)

REVISION_NOTES_FILE = typer.Option(
    None,
    "--revision-notes-file",
    "-n",
    help="""
        Path to the file that defines the revision notes for this operation, supports `md`, and `txt` files only.
    """,
)



#
# # Copyright © 2024 Peak AI Limited. or its affiliates. All Rights Reserved.
# #
# # Licensed under the Apache License, Version 2.0 (the "License"). You
# # may not use this file except in compliance with the License. A copy of
# # the License is located at:
# #
# # https://github.com/PeakBI/peak-sdk/blob/main/LICENSE
# #
# # or in the "license" file accompanying this file. This file is
# # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# # ANY KIND, either express or implied. See the License for the specific
# # language governing permissions and limitations under the License.
# #
# # This file is part of the peak-sdk.
# # see (https://github.com/PeakBI/peak-sdk)
# #
# # You should have received a copy of the APACHE LICENSE, VERSION 2.0
# # along with this program. If not, see <https://apache.org/licenses/LICENSE-2.0>
#
"""Peak Tenants service commands."""
from typing import Optional

import typer
from peak.cli.args import OUTPUT_TYPES, PAGING
from peak.constants import OutputTypes, OutputTypesNoTable
from peak.output import Writer
from peak.resources.tenants import Tenant

app = typer.Typer(
    help="Manage tenant settings and quota.",
    short_help="Create and manage Tenant Settings.",
)

_ENTITY_TYPE = typer.Option(
    ...,
    help="Entity type to be used in this operation (e.g. - `workflow`, `webapp`, `api-deployment`).",
)

_DATA_STORE_TYPE = typer.Option(
    None,
    help="Data store type. The only allowed values is data-warehouse.",
)


@app.command(
    "list-instance-options",
    short_help="List tenant instance options.",
)
def list_instance_options(
    ctx: typer.Context,
    entity_type: str = _ENTITY_TYPE,
    paging: Optional[bool] = PAGING,  # noqa: ARG001
    output_type: Optional[OutputTypes] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***List*** all available instance options for a tenant.

    \b
    📝 ***Example Usage:***<br/>
    ```bash
    peak tenants list-instance-options --entity-type workflow
    ```

    \b
    🆗 ***Response:***
    ```json
    {
      "data": [
        {
          "cpu": 125,
          "gpu": null,
          "gpuMemory": null,
          "id": 20,
          "instanceClass": "General Purpose",
          "memory": 125,
          "name": "Pico (0.125CPU, 0.125GB RAM)",
          "provider": "k8s",
          "providerInstanceId": null
        }
      ]
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/quota/api-docs/index.htm#/settings/get_api_v1_settings_tenant_instance_options)
    """
    tenants_client: Tenant = ctx.obj["client"]
    writer = Writer()

    with writer.pager():
        response = tenants_client.list_instance_options(entity_type=entity_type)
        writer.write(response)


@app.command(
    short_help="Get credentails for a data store.",
)
def get_credentials(
    ctx: typer.Context,
    data_store_type: Optional[str] = _DATA_STORE_TYPE,
    output_type: Optional[OutputTypesNoTable] = OUTPUT_TYPES,  # noqa: ARG001
) -> None:
    """***Get*** credentials for the given data store type.

    \b
    📝 ***Example Usage:***<br/>
    ```bash
    peak tenants get-credentials --data-store-type data-warehouse
    ```

    \b
    🆗 ***Response:***
    ```json
    {
      "application": "application",
      "connectionString": "snowflake://host/database?authenticator=OAUTH&token=generated-access-token",
      "integration": "integration_name",
      "port": 443,
      "role": "role_name",
      "schema": "schema",
      "warehouse": "warehouse",
      "accessToken": "generated-access-token",
      "authType": "oauth",
      "database": "database",
      "host": "host",
      "dataWarehouseType": "snowflake"
    }
    ```

    🔗 [**API Documentation**](https://service.peak.ai/connections/api-docs/index.htm#/connections/get_api_v1_connections_credentials)
    """
    tenants_client: Tenant = ctx.obj["client"]
    writer = Writer()

    with writer.pager():
        response = tenants_client.get_credentials(data_store_type=data_store_type)
        writer.write(response)
